# -*- python -*-
# $RCSfile: introPage.py,v $
# $Revision: 1.9 $
# $Author: langer $
# $Date: 2004/09/24 19:49:29 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common.IO.GUI import oofGUI
from gtk import *

from common.IO.words import words
# words['Credits'], words['Disclaimer'], and words['Copyright']
# are defined in common.IO.

words['Welcome'] = """

                          Welcome to OOF2!

OOF2 performs physical computations on microstructures, starting from a micrograph or other image of the microstructure.  The "Task" menu above brings up pages that lead you through the required steps.

Here is an extremely simplified description of the process, just to get you started.  For more details and examples, see the Tutorials in the Help menu.

First, create a Microstructure, which is OOF2's basic data type. A Microstructure is a map which assigns Materials to pixels.  A Microstructure can contain Images.  You can select pixels in an Image and assign Materials to those pixels in the Microstructure.

After creating a Microstructure and assigning Materials to it, you need to create a Skeleton.  A Skeleton defines the geometry of a finite element mesh, specifying node positions and element edges only. It does not specify element type, equations, or boundary conditions. The Skeleton Modification tools allow you to adapt the Skeleton to the geometry of your Microstructure.  A Microstructure can contain more than one Skeleton.

The FE Mesh page creates a real finite element mesh from a Skeleton by specifying the types of elements to use.  A Skeleton can have more than one Mesh.

The Fields page determines what physical quantities are defined on the mesh (eg, Temperature).  Fields must be defined before they are given values.  Fields must be activated before they can be solved for.

The Equations page determines which equations will be solved. The Boundary Conditions page sets the boundary conditions (surprise!), and the Solver page finds the solution.  Finally, the Analysis and Boundary Analysis pages evaluate the results.

Graphics windows may be opened from the Windows menu.  They can be used to view Image, Microstructures, Skeletons, and Meshes, and to interactively operate on them.  The behavior of a graphics window is determined by the currently selected toolbox, which can be changed with the pull-down menu in the left pane. 

"""

####################
   
font = load_font("-*-fixed-*-*-*-*-*-120-*-*-*-*-iso8859-*")

class IntroPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Introduction", ordering=0,
                                 tip="Welcome to OOF2!")
        vbox = GtkVBox()
        self.gtk.add(vbox)
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_NEVER, POLICY_ALWAYS)
        vbox.add(scroll, expand=1, fill=1)
        self.textarea = GtkText()
        scroll.add(self.textarea)
        self.textarea.set_editable(0)
        self.textarea.set_word_wrap(1)

        buttonbox = GtkHButtonBox()
        vbox.add(buttonbox, expand=0, fill=0)

        self.labels = ['Welcome', 'Credits', 'Copyright', 'Disclaimer']
        self.buttons = [GtkToggleButton(x) for x in self.labels]
        for button, label in zip(self.buttons, self.labels):
            buttonbox.add(button)
            button.connect('clicked', self.buttonCB, label)

        self.buttons[0].set_active(1)

    def buttonCB(self, button, which):
        if button.get_active():
            for button, label in zip(self.buttons, self.labels):
                if label == which:
                    button.set_sensitive(0)
                else:
                    button.set_sensitive(1)
                    button.set_active(0)
            self.textarea.freeze()
            self.textarea.set_point(0)
            self.textarea.forward_delete(self.textarea.get_length())
            self.textarea.insert(font, None, None, words[which])
            self.textarea.thaw()
            
IntroPage()

    
