# -*- python -*-
# $RCSfile: pinnodesPage.py,v $
# $Revision: 1.2 $
# $Author: langer $
# $Date: 2005/02/11 20:01:05 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common import debug
from common import guitop
from common import labeltree
from common import utils
from common.IO import mainmenu
from common.IO import whoville
from common.IO.GUI import oofGUI
from common.IO.GUI import parameterwidgets
from common.IO.GUI import regclassfactory
from common.IO.GUI import whowidget
from engine import skeletoncontext
from engine.IO import pinnodes
from engine import pinnodesmodifier
from gtk import *
import types, string

bold = load_font("-*-fixed-*-*-*-*-*-120-*-*-*-*-iso8859-*")  # gtk stuff

# Define some convenience variables.
OOF = mainmenu.OOF

class PinNodesPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Pin Nodes", ordering=120.1,
                                 tip='Pin and unpin nodes')

        mainbox = GtkVBox(spacing=2)
        self.gtk.add(mainbox)

        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        self.skelwidget = whowidget.WhoWidget(whoville.getClass('Skeleton'),
                                              callback=self.select_skeletonCB)
        label = GtkLabel('Microstructure=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.skelwidget.gtk[0], expand=0, fill=0)
        label = GtkLabel('Skeleton=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.skelwidget.gtk[1], expand=0, fill=0)

        mainpane = GtkHPaned()
        mainbox.add(mainpane)

        # Pinned nodes status in the left half of the main pane
        pnsframe = GtkFrame("Pinned Nodes Status")
        self.datascroll = GtkScrolledWindow()
        pnsframe.add(self.datascroll)
        self.datascroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.psdata = GtkText()
        self.psdata.set_editable(0)
        self.psdata.set_word_wrap(TRUE)
        self.datascroll.add_with_viewport(self.psdata)
        mainpane.pack1(pnsframe, resize=1, shrink=0)
        
        # Pin nodes method
        modframe = GtkFrame("Pin Nodes Methods")
        modbox = GtkVBox()  # will have "methods" and "buttons"
        vbox = GtkVBox()
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.pinModFactory = regclassfactory.RegisteredClassFactory(
            pinnodesmodifier.PinNodesModifier.registry,
            title="Method:", scope=self)
        scroll.add_with_viewport(self.pinModFactory.gtk)
        vbox.add(scroll)
        modbox.pack_start(vbox, TRUE, TRUE, padding=2)

        # buttons
        bbox = GtkVBox()
        bbox1 = GtkHBox()
        self.okbutton = GtkButton("OK")
        self.okbutton.connect("clicked", self.okCB)
        guitop.top().tooltip(self.okbutton, "Pin nodes with the selected method.")
        self.undobutton = GtkButton("Undo")
        self.undobutton.connect("clicked", self.undoCB)
        guitop.top().tooltip(self.undobutton, "Undo the latest action.")
        self.redobutton = GtkButton("Redo")
        self.redobutton.connect("clicked", self.redoCB)
        guitop.top().tooltip(self.redobutton, "Redo the latest undone action.")
        bbox1.pack_start(self.undobutton, TRUE, TRUE, padding=2)
        bbox1.pack_start(self.okbutton, TRUE, TRUE, padding=2)
        bbox1.pack_end(self.redobutton, TRUE, TRUE, padding=2)
        bbox.pack_start(bbox1, TRUE, TRUE, padding=2)

        bbox2 = GtkHBox()
        self.unpinallbutton = GtkButton("UnPinAll")
        self.unpinallbutton.connect("clicked", self.unpinallCB)
        guitop.top().tooltip(self.unpinallbutton, "Unpin all the pinned nodes.")
        self.invertbutton = GtkButton("Invert")
        self.invertbutton.connect("clicked", self.invertCB)
        guitop.top().tooltip(self.invertbutton, "Invert - pin the unpinned and unpin the pinned.")
        bbox2.pack_start(self.unpinallbutton, TRUE, TRUE, padding=2)
        bbox2.pack_end(self.invertbutton, TRUE, TRUE, padding=2)
        bbox.pack_end(bbox2, TRUE, TRUE, padding=2)
        modbox.pack_end(bbox, FALSE, FALSE, padding=2)
        
        modframe.add(modbox)
        mainpane.pack2(modframe, resize=1, shrink=0)

        # Switchboard callbacks
        switchboard.requestCallback(('who changed', 'Skeleton'),
                                    self.changeSkeleton)
        switchboard.requestCallback(("new who", 'Skeleton'), self.newSkeleton)
        switchboard.requestCallback("new pinned nodes", self.newNodesPinned)
        switchboard.requestCallback(self.skelwidget, self.skel_update)

        # Initial sensitizing
        self.sensitize()

    # Switchboard callbacks
    def changeSkeleton(self, skelcontext):
        try:
            path = skelcontext.path()
        except KeyError:
            return
        if self.skelwidget.get_value()==path:
            self.set_state(path)

    def newSkeleton(self, skeleton):
        self.set_state(skeleton)

    def newNodesPinned(self, pinnednodes, skelcontext):
        try:
            path = skelcontext.path()
        except KeyError:
            return
        if self.skelwidget.get_value()==path:
            self.update(path)

    def skel_update(self, *args, **kwargs):  # Switchboard "self.skelwidget"
        skeleton = self.skelwidget.get_value()
        self.update(skeleton)

    # setting the state of "self.skelwidget"
    def set_state(self, skeleton):
        if labeltree.makePath(skeleton) == \
               labeltree.makePath(self.skelwidget.get_value()):
            self.update(skeleton)
        else:
            self.skelwidget.set_value(skeleton)
            # set_value triggers the "self.skelwidget" switchboard
            # callback, which calls "self.update".
            
    # update pinned nodes status
    def update(self, skeleton):
        skelpath = labeltree.makePath(skeleton)
        self.psdata.backward_delete(self.psdata.get_length())
        if skelpath[-1]:  # path is complete
            skelcontext = skeletoncontext.skeletonContexts[skeleton]
            skel = skelcontext.getObject()
            nnodes = skel.nnodes()
            npinned = skelcontext.pinnednodes.npinned()
            self.psdata.insert_defaults("No. of Total Nodes: %d\n" % nnodes)
            self.psdata.insert_defaults("No. of Pinned Nodes: %d\n" % npinned)
        self.sensitize()

    def select_skeletonCB(self, *args):
        self.set_state(self.skelwidget.get_value())

    def currentSkelContext(self):
        try:
            return skeletoncontext.skeletonContexts[self.skelwidget.get_value()]
        except KeyError:
            return None

    def currentSkelName(self):
        return self.skelwidget.get_value()
    
    def sensitize(self):
        skelcontext = self.currentSkelContext()
        if not not skelcontext:
            n = skelcontext.pinnednodes.npinned()
        self.okbutton.set_sensitive(not not skelcontext)
        self.undobutton.set_sensitive(not not skelcontext and \
                                      skelcontext.pinnednodes.undoable())
        self.redobutton.set_sensitive(not not skelcontext and \
                                      skelcontext.pinnednodes.redoable())
        self.unpinallbutton.set_sensitive(not not skelcontext and n > 0)
        self.invertbutton.set_sensitive(not not skelcontext)


    def okCB(self, *args):
        modmethod = self.pinModFactory.getRegistration()
        if modmethod is not None:
            self.pinModFactory.set_defaults()
            menuitem = getattr(mainmenu.OOF.Skeleton.PinNodes,
                               utils.space2underscore(modmethod.name()))
            menuitem.callWithDefaults(skeleton=self.currentSkelName())

    def undoCB(self, *args):
        mainmenu.OOF.Skeleton.PinNodes.Undo(skeleton=self.currentSkelName())
    
    def redoCB(self, *args):
        mainmenu.OOF.Skeleton.PinNodes.Redo(skeleton=self.currentSkelName())

    def unpinallCB(self, *args):
        mainmenu.OOF.Skeleton.PinNodes.UnpinAll(skeleton=self.currentSkelName())

    def invertCB(self, *args):
        mainmenu.OOF.Skeleton.PinNodes.Invert(skeleton=self.currentSkelName())


# Create the page.
pinnodesPage = PinNodesPage()
