# -*- python -*-
# $RCSfile: meshparameters.py,v $
# $Revision: 1.37 $
# $Author: reida $
# $Date: 2005/02/07 19:37:44 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Special file for the definitions of the parameters corresponding to
# mesh objects.  These principally exist so that they can have
# widgets, which can auto-update with new lists of the relevant object
# when the GUI switches meshes.

from SWIG.engine import equation
from SWIG.engine import field
from SWIG.engine import flux
from common import debug
from common import utils
from common.IO import parameter
from engine import component
import struct

class FieldParameter(parameter.ObjParameter, parameter.ParameterBase):
    def __init__(self, name, value=None, default=None, tip=None, outofplane=0):
        self.outofplane = outofplane    # allow out-of-plane fields as values?
        parameter.ParameterBase.__init__(self, name, value, default, tip)
    def clone(self):
        return self.__class__(self.name, self.value, self.default, self.tip,
                              self.outofplane)
    
    def checker(self, x):
        if not isinstance(x, field.FieldPtr):
            parameter.raiseTypeError(x, "Field")
        if not self.outofplane and not isinstance(x, field.CompoundFieldPtr):
            parameter.raiseTypeError(x, "CompoundField")
            
    def incomputable(self, context):
        try:
            mesh = context.mesh()
        except AttributeError:
            return "FieldParameter is only computable on a Mesh."
        if not mesh:
            return "Mesh is not defined."
        incomp = parameter.ParameterBase.incomputable(self, context)
        if incomp:
            return incomp
        if not self.value.is_defined(mesh):
            return "Field '%s' is not defined on the mesh." % self.value.name()
        return 0
    def valueDesc(self):
        return "A <link linkend='Section:Concepts:Mesh:Field'><classname>Field</classname></link> object."

class FieldAnywayParameter(FieldParameter):
    # Never is incomputable.  See Element::outputFieldsAnyway.
    def incomputable(self, context):
        return 0

class TwoVectorFieldParameter(FieldParameter):
    def checker(self, x):
        if not isinstance(x, field.TwoVectorFieldPtr):
            parameter.raiseTypeError(x, "TwoVectorField")
    def valueDesc(self):
        return "A two component vector <link linkend='Section:Concepts:Mesh:Field'><classname>Field</classname></link>."
    
class FluxParameter(parameter.ObjParameter, parameter.ParameterBase):
    def checker(self, x):
        if not isinstance(x, flux.FluxPtr):
            parameter.raiseTypeError(x, "Flux")
    def valueDesc(self):
        return "A <link linkend='Section:Concepts:Mesh:Flux'><classname>Flux</classname></link> object."

class EquationParameter(parameter.ObjParameter, parameter.ParameterBase):
    def checker(self, x):
        if not isinstance(x, equation.EquationPtr):
            parameter.raiseTypeError(x, "Equation")
    def valueDesc(self):
        return "An <link linkend='Section:Concepts:Mesh:Equation'><classname>Equation</classname></link> object."

class FieldIndexParameter(parameter.StringParameter):
    def valueDesc(self):
        return "A character string representing a field or flux index (eg, <userinput>'x'</userinput>)."

# Special parameters for mesh boundaries -- these have widgets which
# present only the names of the right sorts of boundaries, and only
# those which are of nonzero size.
class MeshBoundaryParameter(parameter.StringParameter):
    def valueDesc(self):
        return "A character string representing a point or edge boundary in a mesh."

class MeshEdgeBdyParameter(MeshBoundaryParameter):
    def valueDesc(self):
        return "A character string representing an edge boundary in a mesh."

class MeshPointBdyParameter(MeshBoundaryParameter):
    def valueDesc(self):
        return "A character string representing an point boundary in a mesh."
    
