# -*- python -*-
# $RCSfile: segmentselectdisplay.py,v $
# $Revision: 1.18 $
# $Author: langer $
# $Date: 2004/11/16 21:52:56 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import color
from common import primitives
from common import registeredclass
from common.IO import display
from common.IO import parameter
from common.IO import xmlmenudump

class SkeletonSegmentSelectionDisplay(display.DisplayMethod):
    def __init__(self, color, line_width):
        self.color = color
        self.line_width = line_width
        display.DisplayMethod.__init__(self)
    def draw(self, gfxwindow, device):
        skel = self.who().resolve(gfxwindow)
        if skel is not None:
            device.set_lineColor(self.color)
            device.set_lineWidth(self.line_width)
            for s in skel.segmentselection.retrieve():
                device.draw_segment(primitives.Segment(s.nodes()[0].position(),
                                                       s.nodes()[1].position()))
    def getTimeStamp(self, gfxwindow):
        return max(self.timestamp,
                   self.who().resolve(gfxwindow).segmentselection.timestamp)
            
# This object should be created via the registration, and not
# directly via the initializer, because the registration creation
# method gives it a timestamp.

segmentSelectDisplay = registeredclass.Registration(
    'Selected Segments',
    display.DisplayMethod,
    SkeletonSegmentSelectionDisplay,
    params=[color.ColorParameter('color', color.RGBColor(0.13, 0.93, 0.25),
                                 tip="Color for the selected segments."),
            parameter.IntRangeParameter('line_width', (0,10), 2,
                                        tip="Line width.")],
    ordering=2.1,
    layerordering=display.SemiLinear(4),
    whoclasses=('Skeleton',),
    tip="Display the currently selected segments.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/IO/segmentselectdisplay.xml')
    )
                                                

display.predefineLayer('Skeleton', '<topmost>', segmentSelectDisplay())
