# -*- python -*-
# $RCSfile: preconditioner_.py,v $
# $Revision: 1.20 $
# $Author: langer $
# $Date: 2005/02/03 21:40:04 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import cregisteredclass
from common.IO import parameter
from types import *

cregisteredclass.registerCClass(PreconditionerWrapPtr)

PreconditionerWrapPtr.tip = "Available preconditioners for finite element matrix in <application>OOF2</application>."
PreconditionerWrapPtr.discussion = "<para>TODO: Add discussion.</para>"

cregisteredclass.registerCClass(UnitPreconditionerWrapPtr)

UnitPreconditionerWrapPtr.tip = "Available core preconditioners for finite element matrix in <application>OOF2</application>."
UnitPreconditionerWrapPtr.discussion = "<para>TODO: Add discussion.</para>"

# Special registration that registers the indicated class as both a
# Preconditioner and a UnitPreconditioner.
class UnitPreconditionerRegistration(cregisteredclass.Registration):
    def __init__(self, name, subclass, ordering,
                 params=[], secret=0, **kwargs):
        cregisteredclass.Registration.__init__(
            self, name, (PreconditionerWrapPtr, UnitPreconditionerWrapPtr),
            subclass, ordering, params, secret, **kwargs)

iluPreconditioner = UnitPreconditionerRegistration(
    "ILU",
    ILUPreconditioner,
    ordering=100,
    tip="Incomplete LU-factorization",
    params=[])

UnitPreconditionerRegistration(
    "IC",
    ICPreconditioner,
    ordering=200,
    params=[],
    tip="Incomplete Cholesky preconditioner is appropriate only for microstructures with symmetric, positive definite stiffness matrices")

UnitPreconditionerRegistration(
    "Null",
    UnPreconditioner,
    ordering=300,
    params=[],
    tip="Be bold (or foolhardy) and attempt to solve the mesh without a preconditioner")

UnitPreconditionerRegistration(
    "Jacobi",
    JacobiPreconditioner,
    ordering=500,
    params=[],
    tip="Jacobi preconditioner is the inverse of the diagonal part of the matrix, and is inexpensive to build.")


# Block preconditioner is composite, so it uses the ordinary
# registration process.
blockPreconditioner = cregisteredclass.Registration(
    "Block",
    PreconditionerWrapPtr,
    BlockPreconditioner,
    ordering=1,
    params=[parameter.RegisteredParameter("inplane", UnitPreconditionerWrapPtr,
                                          tip=parameter.emptyTipString)],
    tip="An experimental preconditioner that knows about the structure of the finite element matrix.")
