# This file was created automatically by SWIG.
import anisoc
from SWIG.engine.property.elasticity.homogeneous import HomogeneousElasticityPtr
from SWIG.engine.property.elasticity.cijkl import CijklPtr
class AnisoElasticityPtr(HomogeneousElasticityPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self,anisoc=anisoc):
        if self.thisown == 1 :
            anisoc.delete_AnisoElasticity(self)
    def crystal_cijkl(self, *_args, **_kwargs):
        val = apply(anisoc.AnisoElasticity_crystal_cijkl,(self,) + _args, _kwargs)
        if val: val = CijklPtr(val) ; val.thisown = 1
        return val
    def __repr__(self):
        return "<C AnisoElasticity instance at %s>" % (self.this,)
class AnisoElasticity(AnisoElasticityPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(anisoc.new_AnisoElasticity,_args,_kwargs)
        self.thisown = 1






#-------------- FUNCTION WRAPPERS ------------------



#-------------- VARIABLE WRAPPERS ------------------



#-------------- USER INCLUDE -----------------------

# -*- python -*-
# $RCSfile: aniso_.py,v $
# $Revision: 1.33 $
# $Author: redwing $
# $Date: 2005/02/02 15:51:38 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Definitions of anisotropic elastic properties

# Import definitions of other composite objects.
# from SWIG.engine.properties import PropertyRegistration

from SWIG.engine.property.elasticity import cijkl
# Module cijkl has a name-collision with the cijkl parameter
# in the __init__, but both names make sense in their respective
# contexts, so locally alias cijkl module.
cijkl_module = cijkl

from common.IO import parameter
from engine import problem
from engine import propertyregistration 
from engine.IO import anisocijkl

Parameter = parameter.Parameter
PropertyRegistration = propertyregistration.PropertyRegistration

# CUBIC

class CubicElasticityProp(AnisoElasticity):
    def __init__(self, registration, name, cijkl):
        modulus = cijkl_module.Cijkl()
        # CubicElasticity's cijkl is a convertible parameter,
        # get it in base form before assigning.
        cbase = cijkl.to_base()
        modulus[0,0] = modulus[1,1] = modulus[2,2] = cbase.c11
        modulus[0,1] = modulus[0,2] = modulus[1,2] = cbase.c12
        modulus[3,3] = modulus[4,4] = modulus[5,5] = cbase.c44
        AnisoElasticity.__init__(self, registration, self, name,
                                 modulus)
    # Repr refers to the registered name, not the actual class name,
    # because these things get instanced by the property registration
    # mechanism.
    def __repr__(self):
        cijkl = self.crystal_cijkl()
        return "Cubic(name='%s', cijkl=%s)" % \
               (self.name(), `anisocijkl.CubicRank4TensorCij(cijkl[0,0],
                                                         cijkl[0,1],
                                                         cijkl[3,3])` )

PropertyRegistration(
    'Elasticity:Anisotropic:Cubic', CubicElasticityProp,
    "SWIG.engine.property.elasticity.aniso.aniso",11,
    [anisocijkl.CubicRank4TensorParameter('cijkl',
                                      anisocijkl.CubicRank4TensorCij(c11=1.0,
                                                                 c12=0.5,
                                                                 c44=0.25),
                                          tip=parameter.emptyTipString)],
    fields=[problem.Displacement],
    fluxes=[problem.Stress],
    outputs=["Energy"],
    propertyType="Elasticity",
    tip="Cubic linear elasticity."
    )


# HEXAGONAL 

class HexagonalElasticityProp(AnisoElasticity):
    def __init__(self, registration, name, cijkl):
        modulus = cijkl_module.Cijkl()
        modulus[0,0] = modulus[1,1] = cijkl.c11
        modulus[2,2] = cijkl.c33
        modulus[0,1] = cijkl.c12
        modulus[0,2] = modulus[1,2] = cijkl.c13
        modulus[3,3] = modulus[4,4] = cijkl.c44
        modulus[5,5] = 0.5*(cijkl.c11 - cijkl.c12)
        AnisoElasticity.__init__(self, registration, self, name,
                                 modulus)
    def __repr__(self):
        c = self.crystal_cijkl()
        return "Hexagonal(name='%s', cijkl=%s)" % \
               (self.name(),
                `anisocijkl.HexagonalRank4TensorCij(c11=c[0,0], c12=c[0,1],
                                                c13=c[0,2], c33=c[2,2],
                                                c44=c[3,3])` )

PropertyRegistration(
    'Elasticity:Anisotropic:Hexagonal',
    HexagonalElasticityProp,
    "SWIG.engine.property.elasticity.aniso.aniso",12,
    [anisocijkl.HexagonalCijklParameter(
    'cijkl', anisocijkl.HexagonalRank4TensorCij(c11=1.0,
                                            c12=0.5, c13=0.0,
                                                c33=0.0, c44=0.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Displacement],
    fluxes=[problem.Stress],
    outputs=["Energy"],
    propertyType="Elasticity",
    tip="Hexagonal linear elasticity."
    )

# TETRAGONAL

class TetragonalElasticityProp(AnisoElasticity):
    def __init__(self, registration, name, cijkl):
        modulus = cijkl_module.Cijkl()
        modulus[0,0] = modulus[1,1] = cijkl.c11
        modulus[0,1] = cijkl.c12
        modulus[0,2] = modulus[1,2] = cijkl.c13
        modulus[2,2] = cijkl.c33
        modulus[3,3] = modulus[4,4] = cijkl.c44
        modulus[5,5] = cijkl.c66
        modulus[0,5] = cijkl.c16
        modulus[1,5] = -cijkl.c16
        AnisoElasticity.__init__(self, registration, self, name,
                                 modulus)
    def __repr__(self):
        c = self.crystal_cijkl()
        return "Tetragonal(name='%s', cijkl=%s)" % \
               (self.name(), `anisocijkl.TetragonalRank4TensorCij(
            c11=c[0,0], c12=c[0,1], c13=c[0,2], c33=c[2,2],
            c44=c[3,3], c66=c[5,5], c16=c[0,5])` )




PropertyRegistration(
    'Elasticity:Anisotropic:Tetragonal',
    TetragonalElasticityProp,
    "SWIG.engine.property.elasticity.aniso.aniso",13,
    [anisocijkl.TetragonalCijklParameter(
    'cijkl', anisocijkl.TetragonalRank4TensorCij(
    c11=1.0, c12=0.5, c13=0.5, c33=1.0, c44=0.0, c66=0.0, c16=0.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Displacement],
    fluxes=[problem.Stress],
    outputs=["Energy"],
    propertyType="Elasticity",
    tip="Tetragonal linear elasticity."
    )


# TRIGONAL-A

class TrigonalAElasticityProp(AnisoElasticity):
    def __init__(self, registration, name, cijkl):
        modulus = cijkl_module.Cijkl()
        modulus[0,0] = modulus[1,1] = cijkl.c11
        modulus[0,1] = cijkl.c12
        modulus[0,2] = modulus[1,2] = cijkl.c13
        modulus[2,2] = cijkl.c33
        modulus[3,3] = modulus[4,4] = cijkl.c44
        modulus[5,5] = 0.5*(cijkl.c11 - cijkl.c12)
        
        modulus[0,3] = cijkl.c14
        modulus[1,3] = -cijkl.c14
        modulus[4,5] = cijkl.c14
        
        modulus[0,4] = cijkl.c15
        modulus[1,4] = -cijkl.c15
        modulus[3,5] = -cijkl.c15
        AnisoElasticity.__init__(self, registration, self, name,
                                 modulus)
    def __repr__(self):
        c = self.crystal_cijkl()
        return "TrigonalA(name='%s', cijkl=%s)" % \
               (self.name(), `anisocijkl.TrigonalARank4TensorCij(
            c11=c[0,0], c12=c[0,1], c13=c[0,2], c33=c[2,2],
            c44=c[3,3], c14=c[0,3], c15=c[0,4])` )


PropertyRegistration(
    'Elasticity:Anisotropic:TrigonalA',
    TrigonalAElasticityProp,
    "SWIG.engine.property.elasticity.aniso.aniso",14,
    [anisocijkl.TrigonalACijklParameter('cijkl',
                                        anisocijkl.TrigonalARank4TensorCij(
    c11=1.0, c12=0.5, c13=0.5, c33=1.0, c44=0.25, c14=0.0, c15=0.0),
                                        tip=parameter.emptyTipString)],
    fields=[problem.Displacement],
    fluxes=[problem.Stress],
    outputs=["Energy"],
    propertyType="Elasticity",
    tip="Trigonal-A linear elasticity.")


# TRIGONAL-B

class TrigonalBElasticityProp(AnisoElasticity):
    def __init__(self, registration, name, cijkl):
        modulus = cijkl_module.Cijkl()
        modulus[0,0] = modulus[1,1] = cijkl.c11
        modulus[0,1] = cijkl.c12
        modulus[0,2] = modulus[1,2] = cijkl.c13
        modulus[2,2] = cijkl.c33
        modulus[3,3] = modulus[4,4] = cijkl.c44
        modulus[5,5] = 0.5*(cijkl.c11 - cijkl.c12)

        modulus[0,3] = cijkl.c14
        modulus[1,3] = -cijkl.c14
        modulus[4,5] = cijkl.c14
        AnisoElasticity.__init__(self, registration, self, name,
                                 modulus)
    def __repr__(self):
        c = self.crystal_cijkl()
        return "TrigonalB(name='%s', cijkl=%s)" % \
               (self.name(), `anisocijkl.TrigonalBRank4TensorCij(
            c11=c[0,0], c12=c[0,1], c13=c[0,2], c33=c[2,2], c44=c[3,3],
            c14=c[0,3])` )


PropertyRegistration('Elasticity:Anisotropic:TrigonalB',
                     TrigonalBElasticityProp,
                     "SWIG.engine.property.elasticity.aniso.aniso",15,
                     [anisocijkl.TrigonalBCijklParameter(
    'cijkl', anisocijkl.TrigonalBRank4TensorCij(
    c11=1.0, c12=0.5,c13=0.5,c33=1.0,c44=0.25,c14=0.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Displacement], fluxes=[problem.Stress],
    outputs=["Energy"],
    propertyType="Elasticity",
                     tip="Trigonal-B linear elasticity.")


# ORTHORHOMBIC

class OrthorhombicElasticityProp(AnisoElasticity):
    def __init__(self, registration, name, cijkl):
        modulus = cijkl_module.Cijkl()
        modulus[0, 0] = cijkl.c11
        modulus[0, 1] = cijkl.c12
        modulus[0, 2] = cijkl.c13
        modulus[1, 1] = cijkl.c22
        modulus[1, 2] = cijkl.c23
        modulus[2, 2] = cijkl.c33
        modulus[3, 3] = cijkl.c44
        modulus[4, 4] = cijkl.c55
        modulus[5, 5] = cijkl.c66
        AnisoElasticity.__init__(self, registration, self, name,
                                 modulus)
    def __repr__(self):
        c = self.crystal_cijkl()
        return "Orthorhombic(name='%s', cijkl=%s)"  % \
               (self.name(), `anisocijkl.OrthorhombicRank4TensorCij(
                c11=c[0,0], c12=c[0,1], c13=c[0,2],
                c22=c[1,1], c23=c[1,2], c33=c[2,2],
                c44=c[3,3], c55=c[4,4], c66=c[5,5])` )


PropertyRegistration(
    'Elasticity:Anisotropic:Orthorhombic',
    OrthorhombicElasticityProp,
    "SWIG.engine.property.elasticity.aniso.aniso",16,
    [anisocijkl.OrthorhombicCijklParameter(
    'cijkl',
    anisocijkl.OrthorhombicRank4TensorCij(c11=1.0, c12=0.5, c13=0.5,
                                      c22=1.0, c23=0.5, c33=1.0,
                                      c44=0.25, c55=0.25, c66=0.25),
    tip=parameter.emptyTipString)],
    fields=[problem.Displacement],
    fluxes=[problem.Stress],
    outputs=["Energy"],
    propertyType="Elasticity",
    tip="Orthorhombic linear elasticity.")

# MONOCLINIC

class MonoclinicElasticityProp(AnisoElasticity):
    def __init__(self, registration, name, cijkl):
        modulus = cijkl_module.Cijkl()
        modulus[0, 0] = cijkl.c11
        modulus[0, 1] = cijkl.c12
        modulus[0, 2] = cijkl.c13
        modulus[0, 4] = cijkl.c15
        modulus[1, 1] = cijkl.c22
        modulus[1, 2] = cijkl.c23
        modulus[1, 4] = cijkl.c25
        modulus[2, 2] = cijkl.c33
        modulus[2, 4] = cijkl.c35
        modulus[3, 3] = cijkl.c44
        modulus[3, 5] = cijkl.c46
        modulus[4, 4] = cijkl.c55
        modulus[5, 5] = cijkl.c66
        AnisoElasticity.__init__(self, registration, self, name,
                                 modulus)
    def __repr__(self):
        c = self.crystal_cijkl()
        return "MonoclinicElasticity(name='%s', cijkl=%s)" % \
               (self.name(), `anisocijkl.MonoclinicRank4TensorCij(
            c11=c[0,0], c12=c[0,1], c13=c[0,2], c15=c[0,4],
            c22=c[1,1], c23=c[1,2], c25=c[1,4], c33=c[2,2], c35=c[2,4],
            c44=c[3,3], c46=c[3,5], c55=c[4,4], c66=c[5,5])` )


PropertyRegistration(
    'Elasticity:Anisotropic:Monoclinic',
    MonoclinicElasticityProp,
    "SWIG.engine.property.elasticity.aniso.aniso",17,
    [anisocijkl.MonoclinicCijklParameter(
    'cijkl',
    anisocijkl.MonoclinicRank4TensorCij(c11=1.0, c12=0.5, c13=0.5, c15=0.0,
                                        c22=1.0, c23=0.5, c25=0.0,
                                        c33=0.25, c35=0.0, c44=0.25,
                                        c46=0.0, c55=0.25, c66=0.25),
    tip=parameter.emptyTipString)],
    fields=[problem.Displacement],
    fluxes=[problem.Stress],
    outputs=["Energy"],
    propertyType="Elasticity",
    tip="Monoclinic linear elasticity.")


# TRICLINIC

class TriclinicElasticityProp(AnisoElasticity):
    def __init__(self, registration, name, cijkl):
        modulus = cijkl_module.Cijkl()
        modulus[0,0] = cijkl.c11
        modulus[0,1] = cijkl.c12
        modulus[0,2] = cijkl.c13
        modulus[0,3] = cijkl.c14
        modulus[0,4] = cijkl.c15
        modulus[0,5] = cijkl.c16

        modulus[1,1] = cijkl.c22
        modulus[1,2] = cijkl.c23
        modulus[1,3] = cijkl.c24
        modulus[1,4] = cijkl.c25
        modulus[1,5] = cijkl.c26

        modulus[2,2] = cijkl.c33
        modulus[2,3] = cijkl.c34
        modulus[2,4] = cijkl.c35
        modulus[2,5] = cijkl.c36

        modulus[3,3] = cijkl.c44
        modulus[3,4] = cijkl.c45
        modulus[3,5] = cijkl.c46

        modulus[4,4] = cijkl.c55
        modulus[4,5] = cijkl.c56

        modulus[5,5] = cijkl.c66
        AnisoElasticity.__init__(self, registration, self, name,
                                 modulus)
    def __repr__(self):
        c = self.crystal_cijkl()
        return "Triclinic(name='%s', cijkl=%s)" % \
               (self.name(),
                `anisocijkl.TriclinicRank4TensorCij(
            c11=c[0,0], c12=c[0,1], c13=c[0,2],
            c14=c[0,3], c15=c[0,4], c16=c[0,5],
            c22=c[1,1], c23=c[1,2], c24=c[1,3],
            c25=c[1,4], c26=c[1,5], c33=c[2,2],
            c34=c[2,3], c35=c[2,4], c36=c[2,5],
            c44=c[3,3], c45=c[3,4], c46=c[3,5],
            c55=c[4,4], c56=c[4,5], c66=c[5,5])` )


    
PropertyRegistration(
    'Elasticity:Anisotropic:Triclinic',
    TriclinicElasticityProp,
    "SWIG.engine.property.elasticity.aniso.aniso",18,
    [anisocijkl.TriclinicCijklParameter(
    'cijkl',
    anisocijkl.TriclinicRank4TensorCij(c11=1.0, c12=0.5, c13=0.5, c14=0.0,
                                   c15=0.0, c16=0.0, c22=1.0, c23=0.5,
                                   c24=0.0, c25=0.0, c26=0.0, c33=1.0,
                                   c34=0.0, c35=0.0, c36=0.0, c44=0.25,
                                   c45=0.0, c46=0.0, c55=0.25, c56=0.0,
                                   c66=0.25),
    tip=parameter.emptyTipString)],
    fields=[problem.Displacement],
    fluxes=[problem.Stress],
    outputs=["Energy"],
    propertyType="Elasticity",
    tip="Triclinic linear elasticity.")
