// -*- C++ -*-
// $RCSfile: homogeneous.h,v $
// $Revision: 1.7 $
// $Author: langer $
// $Date: 2004/03/16 16:45:35 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

// "Homogeneous" is a sub-type of elasticity, which while not
// immediately obvious from the name, should be clear from the
// directory in which it is found, and failing that, from this
// comment.  It differs from regular elasticity in that it overrides
// the spatially-dependent cijkl evaluation by calling a new,
// non-spatially-dependent one, which is the right one to use in
// all the descendents. 

#ifndef HOMOGENEOUS_H
#define HOMOGENEOUS_H

#include "engine/property.h"
#include "engine/property/elasticity/elasticity.h"
#include "engine/property/elasticity/cijkl.h"
#include <string>

class Element;
class Flux;
class Material;
class FEMesh;
class Position;
class TwoVectorField;
class SymmetricTensorFlux;
class ElementNodeIterator;

class HomogeneousElasticity : public Elasticity {
public:
  HomogeneousElasticity(PyObject *registry, const std::string &name)
    : Elasticity(name, registry) {}
  virtual ~HomogeneousElasticity() {}
  virtual void precompute() {}

  // Homogeneous elasticity adds the argument-free function call,
  // and sets up the argumentative one to call it -- thus, derived
  // types only have to over-ride cijkl(), and their flux-matrix
  // evaluations will still work.
  virtual const Cijkl cijkl() const=0;
  virtual const Cijkl cijkl(const FEMesh*, const Element*, 
			    const MasterPosition&) const { return cijkl(); }
  // protected:
  // TwoVectorField *displacement;
  // SymmetricTensorFlux *stress_flux;
};

#endif
