# -*- python -*-
# $RCSfile: heatconductivity_.py,v $
# $Revision: 1.24 $
# $Author: langer $
# $Date: 2004/12/23 16:03:16 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from common import utils
from common.IO import parameter
from engine import problem
from engine import propertyregistration
from SWIG.engine import symmmatrix

PropertyRegistration = propertyregistration.PropertyRegistration

heatConductivity = PropertyRegistration(
    'Thermal:Conductivity:Isotropic',
    IsoHeatConductivityProp,
    "SWIG.engine.property.heatconductivity.heatconductivity",
    ordering=98,
    params=[parameter.FloatParameter('kappa',
                                     1.0, tip=parameter.emptyTipString)],
    fields=[problem.Temperature],
    fluxes=[problem.Heat_Flux],
    propertyType="HeatConductivity",
    tip="Isotropic thermal conductivity."
    )

# Reprs refer to the registered name, not the actual class name,
# because properties are instanced via the registry.
def _IsoHeatConductivityProp___repr__(self):
    return "Isotropic(name='%s', kappa=%s)" % \
           (self.name(), `self.kappa()` )

IsoHeatConductivityPropPtr.__repr__ = _IsoHeatConductivityProp___repr__



## TRICLINIC

class TriclinicThermalConductivity(AnisoHeatConductivityProp):
    def __init__(self, registration, name, kij):
        AnisoHeatConductivityProp.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Triclinic(name='%s', kappa=%s)" % \
               (self.name(),`self.kappa()`)


PropertyRegistration(
    'Thermal:Conductivity:Anisotropic:Triclinic', TriclinicThermalConductivity,
    "SWIG.engine.property.heatconductivity.heatconductivity",105,
    [symmmatrix.TriclinicRank2TensorParameter(
    'kappa',
    symmmatrix.TriclinicRank2Tensor(xx=1.0, yy=1.0, zz=1.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Temperature], fluxes=[problem.Heat_Flux],
    propertyType="ThermalConductivity",
    tip="Triclinic thermal conductivity")

   

## MONOCLINIC

class MonoclinicThermalConductivity(AnisoHeatConductivityProp):
    def __init__(self, registration, name, kij):
        AnisoHeatConductivityProp.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Monoclinic(name='%s', kappa=%s)" % \
               (self.name(),`self.kappa()`)


PropertyRegistration(
    'Thermal:Conductivity:Anisotropic:Monoclinic', MonoclinicThermalConductivity,
    "SWIG.engine.property.heatconductivity.heatconductivity",104,
    [symmmatrix.MonoclinicRank2TensorParameter(
    'kappa',
    symmmatrix.MonoclinicRank2Tensor(xx=1.0, yy=1.0, zz=1.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Temperature], fluxes=[problem.Heat_Flux],
    propertyType="ThermalConductivity",
    tip="Monoclinic thermal conductivity.")


## ORTHORHOMBIC
class OrthorhombicThermalConductivity(AnisoHeatConductivityProp):
    def __init__(self, registration, name, kij):
        AnisoHeatConductivityProp.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Orthorhombic(name='%s', kappa=%s)" % \
               (self.name(),`self.kappa()`)


PropertyRegistration(
    'Thermal:Conductivity:Anisotropic:Orthorhombic', OrthorhombicThermalConductivity,
    "SWIG.engine.property.heatconductivity.heatconductivity",103,
    [symmmatrix.OrthorhombicRank2TensorParameter(
    'kappa',
    symmmatrix.OrthorhombicRank2Tensor(xx=1.0, yy=1.0, zz=1.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Temperature], fluxes=[problem.Heat_Flux],
    propertyType="ThermalConductivity",
    tip="Orthorhombic thermal conductivity.")

## For the following crystal structures: tetragonal, trigonal, and hexagonal,
## eventhough the registration is different, the GUI and registration
## parameter objects are the same.

## TETRAGONAL
class TetragonalThermalConductivity(AnisoHeatConductivityProp):
    def __init__(self, registration, name, kij):
        AnisoHeatConductivityProp.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Tetragonal(name='%s', kappa=%s)" % \
               (self.name(),`self.kappa()`)


PropertyRegistration(
    'Thermal:Conductivity:Anisotropic:Tetragonal', TetragonalThermalConductivity,
    "SWIG.engine.property.heatconductivity.heatconductivity",101,
    [symmmatrix.TetragonalRank2TensorParameter(
    'kappa',
    symmmatrix.TetragonalRank2Tensor(xx=1.0, zz=0.5),
    tip=parameter.emptyTipString)],
    fields=[problem.Temperature], fluxes=[problem.Heat_Flux],
    propertyType="ThermalConductivity",
    tip="Tetragonal thermal conductivity.")

## TRIGONAL
class TrigonalThermalConductivity(AnisoHeatConductivityProp):
    def __init__(self, registration, name, kij):
        AnisoHeatConductivityProp.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Trigonal(name='%s', kappa=%s)" % \
               (self.name(),`self.kappa()`)


PropertyRegistration(
    'Thermal:Conductivity:Anisotropic:Trigonal', TrigonalThermalConductivity,
    "SWIG.engine.property.heatconductivity.heatconductivity",102,
    [symmmatrix.TrigonalRank2TensorParameter(
    'kappa',
    symmmatrix.TrigonalRank2Tensor(xx=1.0, zz=0.5),
    tip=parameter.emptyTipString)],
    fields=[problem.Temperature], fluxes=[problem.Heat_Flux],
    propertyType="ThermalConductivity",
    tip="Trigonal thermal conductivity.")

## HEXAGONAL
class HexagonalThermalConductivity(AnisoHeatConductivityProp):
    def __init__(self, registration, name, kij):
        AnisoHeatConductivityProp.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Hexagonal(name='%s', kappa=%s)" % \
               (self.name(),`self.kappa()`)


PropertyRegistration(
    'Thermal:Conductivity:Anisotropic:Hexagonal', HexagonalThermalConductivity,
    "SWIG.engine.property.heatconductivity.heatconductivity",100,
    [symmmatrix.HexagonalRank2TensorParameter(
    'kappa',
    symmmatrix.HexagonalRank2Tensor(xx=1.0, zz=0.5),
    tip=parameter.emptyTipString)],
    fields=[problem.Temperature], fluxes=[problem.Heat_Flux],
    propertyType="ThermalConductivity",
    tip="Hexagonal thermal conductivity.")


## CUBIC
class CubicThermalConductivity(AnisoHeatConductivityProp):
    def __init__(self, registration, name, kij):
        AnisoHeatConductivityProp.__init__(self, registration, self, name,
                                 kij)
    def __repr__(self):
        return "Cubic(name='%s', kappa=%s)" % \
               (self.name(),`self.kappa()`)


PropertyRegistration(
    'Thermal:Conductivity:Anisotropic:Cubic', TetragonalThermalConductivity,
    "SWIG.engine.property.heatconductivity.heatconductivity",99,
    [symmmatrix.CubicRank2TensorParameter(
    'kappa',
    symmmatrix.CubicRank2Tensor(xx=1.0),
    tip=parameter.emptyTipString)],
    fields=[problem.Temperature], fluxes=[problem.Heat_Flux],
    propertyType="ThermalConductivity",
    tip="Cubic thermal conductivity.")


## Thermal BODY forces (sorry Andrew...)

PropertyRegistration(
    'Thermal:HeatSource', HeatSourceProp,
    "SWIG.engine.property.heatconductivity.heatconductivity",150,
    [parameter.FloatParameter('rate', 0.0, tip="Rate of energy increase.")],
    fluxes=[problem.Heat_Flux],
    propertyType="HeatSource",
    tip="A source or sink of heat flux.")

def _HeatSource___repr__(self):
    return "HeatSource(name='%s', rate=%s)" % (self.name(), `self.qdot()`)

HeatSourcePropPtr.__repr__ = _HeatSource___repr__
