# -*- python -*-
# $RCSfile: piezoelectricity_.py,v $
# $Revision: 1.20 $
# $Author: redwing $
# $Date: 2005/02/25 21:52:23 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from SWIG.engine import rank3tensor
from SWIG.engine import symmmatrix
from common import utils
from common.IO import parameter
from engine import problem
from engine import propertyregistration

PropertyRegistration = propertyregistration.PropertyRegistration


######################################################
class C1PiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "C1(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Triclinic:C1',
    C1PiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",300,
    [rank3tensor.C1Rank3TensorParameter('dijk',
                                        rank3tensor.C1Rank3Tensor(d11 = 1, d12 = 1, d13 = 1, d14 = 1, d15 = 1, d16 = 1,
                                                                  d21 = 1, d22 = 1, d23 = 1, d24 = 1, d25 = 1, d26 = 1,
                                                                  d31 = 1, d32 = 1, d33 = 1, d34 = 1, d35 = 1, d36 = 1)
                                        ,
                                        tip="point group 1 piezoelectricity tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C1 piezoelectricity')

######################################################
class C2PiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "C2(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Monoclinic:C2',
    C2PiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",301,
    [rank3tensor.C2Rank3TensorParameter('dijk',
                                        rank3tensor.C2Rank3Tensor(d14 = 1, d16 = 1,
                                                                  d21 = 1, d22 = 1, d23 = 1, d25 = 1, 
                                                                  d34 = 1, d36 = 1) 
                                        ,
                                        tip="point group 2 (with a single two-fold axis) piezoelectricity tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C2 piezoelectricity')

######################################################

class CsPiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "Cs(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Monoclinic:Cs',
    CsPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",302,
    [rank3tensor.CsRank3TensorParameter('dijk',
                                        rank3tensor.CsRank3Tensor(d11 = 1, d12 = 1, d13 = 1, d15 = 1,
                                                                  d24 = 1, d26 = 1,
                                                                  d31 = 1, d32 = 1, d33 = 1, d35 = 1) 
                                        ,
                                        tip="point group m (with a single mirror plane) piezoelectricity tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='Cs piezoelectricity')


######################################################
class D2PiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "D2(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Orthorhombic:D2',
    D2PiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",303,
    [rank3tensor.D2Rank3TensorParameter('dijk',
                                     rank3tensor.D2Rank3Tensor(d14=1.0, d25=1.0, d36=1.0),
                                     tip="222 piezoelectricity tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='D2 piezoelectricity')

######################################################
class C2vPiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "C2v(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Orthorhombic:C2v',
    C2vPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",304,
    [rank3tensor.C2vRank3TensorParameter('dijk',
                                     rank3tensor.C2vRank3Tensor(d15 = 1, d24 = 1, d31 = 1, d32 = 1, d33 = 1),
                                     tip="mm2 piezoelectricity tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C2v piezoelectricity')

######################################################
class C4PiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "C4(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Tetragonal:C4',
    C4PiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",305,
    [rank3tensor.C4Rank3TensorParameter('dijk',
                                     rank3tensor.C4Rank3Tensor(d14 = 1, d15 = 1, d31 = 1, d33 = 1),
                                     tip="Point group 4 piezoelectricity tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C4 piezoelectricity')


######################################################
class C4iPiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "C4i(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Tetragonal:C4i',
    C4iPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",306,
    [rank3tensor.C4iRank3TensorParameter('dijk',
                                     rank3tensor.C4iRank3Tensor(d14 = 1, d15 = 1, d31 = 1, d36 = 1),
                                     tip="Also known as C4h and bar{4} piezoelectric tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C4i piezoelectricity')


######################################################
class D4PiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "D4(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Tetragonal:D4',
    C4iPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",307,
    [rank3tensor.D4Rank3TensorParameter('dijk',
                                     rank3tensor.D4Rank3Tensor(d14 = 1),
                                     tip="Also known as 422 piezoelectric tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C4i piezoelectricity')



######################################################
class C4vPiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "C4v(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Tetragonal:C4v',
    C4vPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",308,
    [rank3tensor.C4vRank3TensorParameter('dijk',
                                     rank3tensor.C4vRank3Tensor(d15=1.0, d31=1.0, d33=1.0),
                                     tip="4mm piezoelectricity tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C4v piezoelectricity')

######################################################
class D2dPiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "D2d(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Tetragonal:D2d',
    D2dPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",309,
    [rank3tensor.D2dRank3TensorParameter('dijk',
                                     rank3tensor.D2dRank3Tensor(d14 = 1, d36 = 1),
                                     tip="Also known as bar{4}2m piezoelectric tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='D2d piezoelectricity')

######################################################
class C3PiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "C3(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Trigonal:C3',
    C3PiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",310,
    [rank3tensor.C3Rank3TensorParameter('dijk',
                                     rank3tensor.C3Rank3Tensor(d11 = 1, d14 = 1, d15 = 1, d22 = 1, d31 = 1, d33 = 1),
                                     tip="Also known as point group 3")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C3 piezoelectricity')

######################################################
class D3PiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "D3(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Trigonal:D3',
    D3PiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",311,
    [rank3tensor.D3Rank3TensorParameter('dijk',
                                     rank3tensor.D3Rank3Tensor(d11 = 1, d14 = 1),
                                     tip="Also known as point group 32")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='D3 piezoelectricity')

######################################################
class C3vPiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "C3v(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Trigonal:C3v',
    C3vPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",312,
    [rank3tensor.C3vRank3TensorParameter('dijk',
                                     rank3tensor.C3vRank3Tensor(d14 = 1, d22 = 1, d31 = 1, d33 = 1),
                                     tip="Also known as point group 3m")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C3v piezoelectricity')



######################################################

class C6PiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "C6(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Hexagonal:C6',
    C6PiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",313,
    [rank3tensor.C6Rank3TensorParameter('dijk',
                                     rank3tensor.C6Rank3Tensor(d14 = 1, d15=1.0, d31=1.0, d33=1.0),
                                     tip="Also known as class 6 point group")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C6 piezoelectricity')


######################################################
class D6iPiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "D6i(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Hexagonal:D6i',
    D6iPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",314,
    [rank3tensor.D6iRank3TensorParameter('dijk',
                                     rank3tensor.D6iRank3Tensor(d11 = 1, d22 = 1),
                                     tip="Also known as point group bar{6}")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='D6i piezoelectricity')


######################################################
class D6PiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "D6(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Hexagonal:D6',
    D6iPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",315,
    [rank3tensor.D6Rank3TensorParameter('dijk',
                                     rank3tensor.D6Rank3Tensor(d14 = 1),
                                     tip="Also known as point group 622")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='D6 piezoelectricity')


######################################################
class C6vPiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "C6v(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Hexagonal:C6v',
    C6vPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",316,
    [rank3tensor.C6vRank3TensorParameter('dijk',
                                     rank3tensor.C6vRank3Tensor(d15=1.0, d31=1.0, d33=1.0),
                                     tip="6mm piezoelectricity tensor")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='C6v piezoelectricity')



######################################################
class D3hPiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "D3h(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Hexagonal:D3h',
    D6iPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",317,
    [rank3tensor.D3hRank3TensorParameter('dijk',
                                     rank3tensor.D3hRank3Tensor(d22 = 1),
                                     tip="Also known as point group bar{6}m2")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='D3h piezoelectricity')


######################################################
class TdPiezoElectricity(AnisotropicPiezoElectricity):
    def __init__(self, registration, name, dijk):
        AnisotropicPiezoElectricity.__init__(self, registration, self, name,
                                 dijk)
    def __repr__(self):
        return "Td(name='%s', dijk=%s)" % \
               (self.name(),`self._dijk()`)


PropertyRegistration(
    'Electric:PiezoElectricity:Cubic:Td',
    D6iPiezoElectricity,
    "SWIG.engine.property.piezoelectricity.piezoelectricity",318,
    [rank3tensor.TdRank3TensorParameter('dijk',
                                     rank3tensor.TdRank3Tensor(d14 = 1),
                                     tip="Also known as point group 23, bar{4}3m, or simply T")],
    fields=[problem.Voltage, problem.Displacement],
    fluxes=[problem.Total_Polarization, problem.Stress],
    outputs=["Strain", "Energy"],
    propertyType="PiezoElectricity",
    tip='D6 piezoelectricity')

