# -*- python -*-
# $RCSfile: rationalwide.py,v $
# $Revision: 1.4 $
# $Author: langer $
# $Date: 2005/02/10 20:06:32 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import registeredclass
from common.IO import parameter
from common.IO import xmlmenudump
from engine import rationalize
from engine import rationalizefunctions
import math

Rationalizer = rationalize.Rationalizer
quadSplit = rationalizefunctions.quadSplit

class QuadSplit(Rationalizer):
    #
    #              D____C
    #              /    |   Find the biggest angle and if it is bigger than
    #             /     |   the specified threshold, split the quad
    #            /      |
    #           /_______|             D____C
    #          A        B   ====>     /\   |
    #                                /  \  |
    #                               /    \ |
    #                              /______\|
    #                             A        B
    #

    def __init__(self, angle):
        self.angle = angle
        self.threshold = math.cos(angle*math.pi/180.0)

    def findAndFix(self, skel, element):
        if element.nnodes() == 4:  # Applies only for quads
            which = element.getBiggestAngle()
            if element.cosCornerAngle(which) <= self.threshold:
                # Returns a list of ProvisionalChange objects
                return self.fix(skel, element, which)
        return []

    def fixAll(self, skel, element):
        if element.nnodes() == 4:
            which = element.getBiggestAngle()
            return self.fix(skel, element, which)
        return []

    def fix(self, skel, element, which):
        return quadSplit(skel, element, which)
    
registeredclass.Registration(
    'Split Wide Quads',
    Rationalizer,
    QuadSplit,
    gerund = 'splitting wide quads',
    ordering=1,
    params=[
    parameter.FloatRangeParameter('angle', (90.0, 180.0, 1.0), value = 150.0,
                                  tip = 'Maximum acceptable interior angle')
    ],
    tip = 'Split quadrilaterals with large interior angles.',
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg-ration_wide.xml'))
