# -*- python -*-
# $RCSfile: imagePage.py,v $
# $Revision: 1.41 $
# $Author: langer $
# $Date: 2004/12/22 03:05:41 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from SWIG.image import oofimage
from common import debug
from common import guitop
from common import labeltree
from common import microstructure
from common import ringbuffer
from common import utils
from common.IO import mainmenu
from common.IO import parameter
from common.IO import whoville
from common.IO import reporter
from common.IO.GUI import chooser
from common.IO.GUI import fileselector
from common.IO.GUI import historian
from common.IO.GUI import oofGUI
from common.IO.GUI import parameterwidgets
from common.IO.GUI import regclassfactory
from common.IO.GUI import whowidget
from image import imagemodifier
from gtk import *
from types import *

font = load_font("-*-fixed-*-*-*-*-*-120-*-*-*-*-iso8859-*")

class ImagePage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Image", ordering=50,
                                 tip='Manipulate Images')

        mainbox = GtkVBox(spacing=2)
        self.gtk.add(mainbox)

        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        label = GtkLabel('Microstructure=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        self.imagewidget = whowidget.WhoWidget(oofimage.imageContexts)
        centerbox.add(self.imagewidget.gtk[0], expand=0, fill=0)
        label = GtkLabel('Image=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.imagewidget.gtk[1], expand=0, fill=0)

        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox(homogeneous=1, spacing=3)
        hbox.add(centerbox, expand=1, fill=0)

        self.loadbutton = GtkButton('Load...')
        centerbox.add(self.loadbutton, expand=1, fill=1)
        self.loadbutton.connect('clicked', self.loadCB)
        guitop.top().tooltip(self.loadbutton,
                             'Load a new image into an existing Microstructure')
        
        self.copybutton = GtkButton('Copy...')
        self.copybutton.connect('clicked', self.copyCB)
        centerbox.add(self.copybutton, expand=1, fill=1)
        guitop.top().tooltip(self.copybutton,
                             'Copy the current image.  The copy can be in the same or a different Microstructure.')

        self.renamebutton = GtkButton('Rename...')
        self.renamebutton.connect('clicked', self.renameCB)
        guitop.top().tooltip(self.renamebutton,
                             'Rename the current image.')
        centerbox.add(self.renamebutton, expand=1, fill=1)

        self.deletebutton = GtkButton('Delete')
        self.deletebutton.connect('clicked', self.deleteCB)
        guitop.top().tooltip(self.deletebutton,
                             'Delete the current image.')
        centerbox.add(self.deletebutton, expand=1, fill=1)

        self.savebutton = GtkButton('Save...')
        self.savebutton.connect('clicked', self.saveCB)
        guitop.top().tooltip(self.savebutton,
                             'Save the current image to a file.')
        centerbox.add(self.savebutton, expand=1, fill=1)

        self.autogroupbutton = GtkButton('Group')
        self.autogroupbutton.connect('clicked', self.autogroupCB)
        centerbox.add(self.autogroupbutton, expand=1, fill=1, padding=2)
        guitop.top().tooltip(self.autogroupbutton, "Create a pixel group in the current image's microstructure for each color pixel in the image.")

        mainpane = GtkHPaned()
        mainbox.add(mainpane)

        frame = GtkFrame('Image Information')
        mainpane.pack1(frame, resize=FALSE, shrink=FALSE)
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        frame.add(scroll)
        self.infoarea = GtkText()
        self.infoarea.set_word_wrap(TRUE)
        scroll.add_with_viewport(self.infoarea)

        frame = GtkFrame('Image Modification')
        mainpane.pack2(frame, resize=1, shrink=1)
        vbox = GtkVBox()
        frame.add(vbox)
        scroll = GtkScrolledWindow()    # scroll window for image mod method
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        vbox.add(scroll)
        self.imageModFactory = regclassfactory.RegisteredClassFactory(
            imagemodifier.ImageModifier.registry, title="Method:")
        scroll.add_with_viewport(self.imageModFactory.gtk)
        self.historian = historian.Historian(self.imageModFactory.set,
                                             self.sensitizeHistory,
                                             setCBkwargs={'interactive':1})
        self.imageModFactory.set_callback(self.historian.stateChangeCB)

        # Prev, OK, and Next buttons
        hbox = GtkHBox()
        vbox.add(hbox, expand=0, fill=0, padding=2)
        self.prevmethodbutton = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkArrow(at=ARROW_LEFT))
        lbox.add(GtkLabel(' Prev'))
        self.prevmethodbutton.add(lbox)
        self.prevmethodbutton.connect('clicked', self.historian.prevCB)
        hbox.add(self.prevmethodbutton, expand=0, fill=0, padding=2)
        guitop.top().tooltip(self.prevmethodbutton,
                             'Recall the previous image modification operation.')
        self.okbutton = GtkButton('OK')
        hbox.add(self.okbutton, expand=1, fill=1, padding=5)
        self.okbutton.connect('clicked', self.okbuttonCB)
        guitop.top().tooltip(self.okbutton,
                  'Perform the image modification operation defined above.')
        self.nextmethodbutton = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkLabel('Next '), expand=0, fill=0)
        lbox.add(GtkArrow(at=ARROW_RIGHT), expand=0, fill=0)
        self.nextmethodbutton.add(lbox)
        self.nextmethodbutton.connect('clicked', self.historian.nextCB)
        hbox.add(self.nextmethodbutton, expand=0, fill=0, padding=2)
        guitop.top().tooltip(self.nextmethodbutton,
                             "Recall the next image modification operation.")

        # Undo and Redo buttons
        hbox = GtkHBox()
        vbox.add(hbox, expand=0, fill=0, padding=2)
        self.undobutton = GtkButton('Undo')
        self.undobutton.connect('clicked', self.undoCB)
        guitop.top().tooltip(self.undobutton,
                             'Undo the latest image modification.')
        hbox.add(self.undobutton, expand=1, fill=0, padding=10)
        self.redobutton = GtkButton('Redo')
        self.redobutton.connect('clicked', self.redoCB)
        guitop.top().tooltip(self.redobutton,
                             'Redo the latest undone image modification.')
        hbox.add(self.redobutton, expand=1, fill=0, padding=10)

        self.sensitize()
        self.sensitizeHistory()
        self.displayImageInfo()
        
        self.sbcallbacks = [
            switchboard.requestCallback(('new who', 'Image'), self.newImageCB),
            switchboard.requestCallback(('remove who', 'Image'), self.rmWhoCB),
            switchboard.requestCallback('modified image', self.modifiedImageCB),
            switchboard.requestCallback(imagemodifier.ImageModifier,
                                        self.updateImageModifiers),
            switchboard.requestCallback(self.imagewidget, self.iWidgetChanged),
            switchboard.requestCallback(('validity', self.imageModFactory),
                                        self.validityChangeCB)
            ]

    def sensitize(self):
        image = self.getCurrentImage()
        selected = image is not None
        self.copybutton.set_sensitive(selected)
        self.renamebutton.set_sensitive(selected)
        self.deletebutton.set_sensitive(selected)
        self.savebutton.set_sensitive(selected)
        self.okbutton.set_sensitive(selected and self.imageModFactory.isValid())
        self.autogroupbutton.set_sensitive(selected)
        self.loadbutton.set_sensitive(self.getCurrentMS() is not None)
        if selected:
            self.undobutton.set_sensitive(image.undoable())
            self.redobutton.set_sensitive(image.redoable())
        else:
            self.undobutton.set_sensitive(0)
            self.redobutton.set_sensitive(0)

    def sensitizeHistory(self):
        debug.mainthreadTest()
        self.nextmethodbutton.set_sensitive(self.historian.nextSensitive())
        self.prevmethodbutton.set_sensitive(self.historian.prevSensitive())

    def validityChangeCB(self, validity):
        self.sensitize()

    def iWidgetChanged(self, interactive):
        self.sensitize()
        self.displayImageInfo()
        
    def displayImageInfo(self):
        debug.mainthreadTest()
        self.infoarea.backward_delete(self.infoarea.get_length())
        imagecontext = self.getCurrentImage()
        if imagecontext:
            image = imagecontext.getObject()
            size = image.sizeInPixels()
            self.infoarea.insert_defaults('Pixel size: %dx%d\n' %
                                          (size.x, size.y))
            size = image.size()
            self.infoarea.insert_defaults('Physical size: %gx%g\n' %
                                          (size.x, size.y))
            self.infoarea.insert_defaults('\nComments:\n%s\n' % image.comment())

    def getCurrentImageName(self):
        path = labeltree.makePath(self.imagewidget.get_value())
        return path[-1]

    def getCurrentMSName(self):
        path = labeltree.makePath(self.imagewidget.get_value())
        return path[0]

    def getCurrentMS(self):
        try:
            return microstructure.microStructures[self.getCurrentMSName()]
        except KeyError:
            pass
        
    def getCurrentImage(self):
        try:
            return oofimage.imageContexts[self.imagewidget.get_value()]
        except KeyError:
            pass
            

    def updateImageModifiers(self):
        self.imageModFactory.update(imagemodifier.ImageModifier.registry)

    def newImageCB(self, whoname):        # switchboard 'new who', 'Image'
        self.imagewidget.set_value(whoname)
        self.sensitize()

    def rmWhoCB(self, whoname):         # switchboard 'remove who'
        self.sensitize()

    ############### GUI callbacks ################

    def loadCB(self, button):
        menuitem = mainmenu.OOF.File.Load.Image
        menuitem.gui_callback(menuitem)

    def copyCB(self, button):
        menuitem = mainmenu.OOF.Image.Copy
        nameparam = menuitem.get_arg('name')
        msparam = menuitem.get_arg('microstructure')
        if parameterwidgets.getParameters(msparam, nameparam):
            menuitem.callWithDefaults(image=self.imagewidget.get_value())

    def renameCB(self, button):
        menuitem = mainmenu.OOF.Image.Rename
        namearg = menuitem.get_arg('name')
        namearg.value = labeltree.makePath(self.imagewidget.get_value())[-1]
        if parameterwidgets.getParameters(namearg, title='Rename image'):
            menuitem.callWithDefaults(image=self.imagewidget.get_value())

    def deleteCB(self, button):
        if reporter.query("Really delete %s?"
                          % self.getCurrentImageName(),
                          "No", default="Yes") == "Yes":
            mainmenu.OOF.Image.Delete(image=self.imagewidget.get_value())
            
    def saveCB(self,button):
        file = fileselector.fileSelector(ident='SaveImage', title='Save Image',
                                         filename=self.getCurrentImageName())
        if file is not None:
            mainmenu.OOF.Image.Save(image=self.imagewidget.get_value(),
                                    filename=file)

    def okbuttonCB(self, button):
        modmeth = self.imageModFactory.getRegistration()
        if modmeth is not None:
            # buildImageModMenu() in imagemenu.py builds the
            # Image.Modify menu items from the members of the
            # ImageModifier RegisteredClass.  The Parameters in the
            # menu items are identically the Parameters in the
            # Registrations.  So copying new values out of the Factory
            # into the Registrations, and invoking the menu item with
            # callWithDefaults() executes the command with the current
            # arguments.
            self.imageModFactory.set_defaults() # copy from gui to Registration
            menuitem = getattr(mainmenu.OOF.Image.Modify, modmeth.name())
            menuitem.callWithDefaults(image=self.imagewidget.get_value())

    def modifiedImageCB(self, imageModifier, imagename): # sb 'modified image'
        # Called whenever the image is changed, either by applying an
        # ImageModifier or the Undo and Redo buttons, in which case
        # the imageModifier arg is None.
        if imageModifier is not None:
            self.historian.record(imageModifier)
        self.sensitize()
        
    def undoCB(self, button):
        mainmenu.OOF.Image.Undo(image=self.imagewidget.get_value())

    def redoCB(self, button):
        mainmenu.OOF.Image.Redo(image=self.imagewidget.get_value())

    def autogroupCB(self, button):
        mainmenu.OOF.Image.AutoGroup(image=self.imagewidget.get_value())
        
ImagePage()
