# -*- python -*-
# $RCSfile: imageIO.py,v $
# $Revision: 1.7 $
# $Author: haan $
# $Date: 2004/11/24 21:52:01 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.image import oofimage
from common import debug
from common import registeredclass
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO import parameter
from common.IO import whoville
from image.IO import imagemenu
import common.microstructure


imgmenu = mainmenu.OOF.LoadData.addItem(oofmenu.OOFMenuItem('Image'))


class ImageData(registeredclass.RegisteredClass):
    registry = []

ImageData.tip = "Available image data types in &oof2;."
ImageData.discussion = "<para>TODO: Add discussion.</para>"

class RGBData8(ImageData):
    def __init__(self, rgbvalues):
        self.rgbvalues = rgbvalues
    def values(self):
        return self.rgbvalues

registeredclass.Registration('RGBData8', ImageData, RGBData8,
                             ordering=0,
                             params=[
    parameter.ListOfUnsignedShortsParameter('rgbvalues',
                                            tip="RGB values.")],
                             tip="RGB image data.")



class GrayData8(ImageData):
    def __init__(self, grayvalues):
        self.grayvalues = grayvalues
    def values(self):
        return self.grayvalues

registeredclass.Registration('GrayData8', ImageData, GrayData8,
                             ordering=1,
                             params=[
    parameter.ListOfUnsignedShortsParameter('grayvalues',
                                            tip="Gray values.")],
                             tip="Gray image data.")



def _newImage(menuitem, name, microstructure, pixels):
    ms = common.microstructure.microStructures[microstructure].getObject()
    image = oofimage.newImageFromData(name, ms.sizeInPixels(), pixels.values())
    image.setSize(ms.size())
    imagemenu.loadImageIntoMS(image, microstructure)

imgmenu.addItem(oofmenu.OOFMenuItem(
    'New',
    callback=_newImage,
    params=[
    parameter.StringParameter('name', tip="Name for the Image."),
    whoville.WhoParameter('microstructure',whoville.getClass('Microstructure'),
                          tip=parameter.emptyTipString),
    parameter.RegisteredParameter('pixels', ImageData,
                                  tip=parameter.emptyTipString)],
    help="Load an Image.",
    discussion="<para>Load an &image; from a saved &micro;.</para>"
    ))



def writeImage(datafile, imagecontext):
    datafile.startCmd(mainmenu.OOF.LoadData.Image.New)
    datafile.argument('name', imagecontext.name())
    datafile.argument('microstructure', imagecontext.getMicrostructure().name())
    datafile.argument('pixels', RGBData8(imagecontext.getObject().getPixels()))
    datafile.endCmd()

# TODO LATER: Allow different image depths, and find a way to store
# only gray values if the image is gray.
